'use strict';

var FlixSlider = function FlixSlider(options) {
  this.init(options);
};

FlixSlider.prototype.init = function init (options) {
  var defaults = {
    flixClass: options.flixClass,
    hoverButtons: false,
    itemsOnScreen: 4,
    itemsToMove: 4,
    sizingClass: 'container',
    pips: false,
    debug: false,
    prevBtnIconHtml: null,
    nextBtnIconHtml: null,
    dragOnDesktop: true,
    responsive: null,
    dragThreshold: 7,
  };

  // update default opions if options passed through
  //
  var returnedOptions = Object.assign(defaults, options);

  var itemsOnScreen = returnedOptions.itemsOnScreen,
    fsItemGutter = 15,
    dragItemsToMove = returnedOptions.itemsOnScreen,
    itemsToMove = returnedOptions.itemsToMove,
    fsArticle = $(returnedOptions.flixClass),
    fsTrack,
    trackIndent,
    numberOfPips,
    fsPips,
    currentPage = 1,
    fsItem,
    containerWidth,
    screenWidth = $(document).width(),
    fsItemWidth,
    fsItemNumber,
    count = 0,
    currentTrackPosition = 0,
    fsButtonPrevIconHTML = returnedOptions.prevBtnIconHtml,
    fsButtonNextIconHTML = returnedOptions.nextBtnIconHtml,
    fsButtonPrev,
    fsButtonNext,
    fsButtonPrevHover,
    fsButtonNextHover,
    hasButtons = false,
    buttonsSet = false,
    dragging = false,
    oldPageX = 0,
    pageXOffset = 50;

  //line up initial track posiion with container
  //
  var setItemWidth = function (items) {
    fsItemWidth = containerWidth / items - fsItemGutter * 2;

    $(fsItem)
      .css('width', fsItemWidth + 'px')
      .css('flex', '0 0 ' + fsItemWidth + 'px')
      .css('margin-left', fsItemGutter + 'px')
      .css('margin-right', fsItemGutter + 'px');
  };

  // Buttons
  //
  var shouldIhaveButtons = function (noItemsOnScreen) {
    if (fsItemNumber > noItemsOnScreen) {
      if (buttonsSet === false) {
        hasButtons = true;
        setUpButtons();
        setButtonVars();
        setButtonType();
      }
    } else {
      if (buttonsSet === true) {
        removeButtons();
      }
    }
  };

  var setUpButtons = function () {
    if (buttonsSet === false) {
      if (fsButtonPrevIconHTML != null || fsButtonNextIconHTML != null) {
        $(fsArticle)
          .append('<button class="fs-prev" tabindex="-1">' + fsButtonPrevIconHTML + '<span class="sr-only">Previous</span></button>')
          .append('<button class="fs-next" tabindex="-1">' + fsButtonNextIconHTML + '<span class="sr-only">Next</span></button>');
      } else {
        $(fsArticle)
          .append('<button class="fs-prev" tabindex="-1"><span class="fs-prev-icon"></span><span class="sr-only">Previous</span></button>')
          .append('<button class="fs-next" tabindex="-1"><span class="fs-next-icon"></span><span class="sr-only">Next</span></button>');
      }

      buttonsSet = true;

      return
    }
  };

  var setButtonVars = function () {
    fsButtonPrev = fsArticle.find('.fs-prev');
    fsButtonNext = fsArticle.find('.fs-next');
    return
  };

  var setButtonType = function () {
    if (returnedOptions.hoverButtons == true) {
      // Button hovers
      //
      $(fsButtonPrev).on('mouseenter', function () {
        fsButtonPrevHover = setTimeout(function callPrev() {
          prevItem(itemsToMove);
          fsButtonPrevHover = setTimeout(callPrev, 1000); // (*)
        }, 100);
      });

      $(fsButtonPrev).on('mouseleave', function () {
        clearTimeout(fsButtonPrevHover);
      });

      $(fsButtonNext).on('mouseenter', function () {
        fsButtonNextHover = setTimeout(function callNext() {
          nextItem(itemsToMove);
          fsButtonNextHover = setTimeout(callNext, 1000); // (*)
        }, 100);
      });

      $(fsButtonNext).on('mouseleave', function () {
        clearTimeout(fsButtonNextHover);
      });
    } else if (returnedOptions.hoverButtons == false) {
      // Button click
      //
      $(fsButtonPrev).on('click', function () {
        prevItem(itemsToMove);
      });
      $(fsButtonNext).on('click', function () {
        nextItem(itemsToMove);
      });
    }

    return
  };

  // set button width to space either side of sizing div
  //
  var calcButtonWidth = function () {
    if (hasButtons === true) {
      fsButtonPrev.css({
        width: trackIndent - fsItemGutter,
        opacity: '0'
      });
      fsButtonNext.css('width', trackIndent - fsItemGutter);
    }
  };

  // prev button function
  //
  var prevItem = function (amountToMove) {
    moveTrack('prev', amountToMove);
  };

  // next button function
  //
  var nextItem = function (amountToMove) {
    moveTrack('next', amountToMove);
  };

  var removeButtons = function () {
    fsButtonPrev.remove();
    fsButtonNext.remove();

    hasButtons = false;
    buttonsSet = false;

    return
  };

  // create the track div
  //
  var setUpTrack = function () {
    $(fsArticle).wrapInner('<div class="fs-track"></div>');
    fsTrack = fsArticle.find('.fs-track');

    if (returnedOptions.debug == true) {
      fsTrack.css('background-color', 'rgba(0,0,255,0.2)');
      fsArticle.append('<div class="fs-touch-event-indicator"></div>');
    }
  };

  // calc number of items and set track width container
  //
  var calcTrackWidth = function () {
    $(fsTrack).css('width', (fsItemWidth + fsItemGutter * 2) * fsItemNumber + 'px');
  };

  // set the sizing div to posision track to
  //
  var setUpSizingDiv = function () {
    $(fsArticle).append('<div class="' + returnedOptions.sizingClass + ' fs-sizer" tabindex="-1"></div> ');
    containerWidth = $('.' + returnedOptions.sizingClass + '.fs-sizer').innerWidth();

    if (returnedOptions.debug == true) {
      $('.fs-sizer').css({
        height: '5px',
        'background-color': 'red'
      });
    }
  };

  var setUpItems = function () {
    $(fsArticle).addClass('flix-slider');
    $(fsArticle)
      .children()
      .wrap('<div class="fs-item"></div>');
    fsItem = fsArticle.find('.fs-item');
    fsItemNumber = $(fsItem).length;
  };

  // set starting point of first item and align with container
  //
  var calcTrackIndent = function () {
    trackIndent = (screenWidth - containerWidth) / 2;
    $(fsTrack).css('left', trackIndent);
  };

  var setArrowKeys = function () {
    $(fsArticle).keydown(function (e) {
      switch (e.which) {
        case 9: // tab
          if (e.shiftKey) {
            prevItem(1); // shift tab
            break
          }
          nextItem(1);
          break

        case 37: // left
          prevItem(itemsToMove);
          break

        case 39: // right
          nextItem(itemsToMove);
          break

        default:
          return
      }
      // e.preventDefault()
    });
  };

  // move track left or right
  //
  var moveTrack = function (direction, amountToMove) {
    if (direction === 'next') {
      if (count + itemsOnScreen < fsItemNumber) {
        // stop items scrolling past right of container
        // calc if track can advance that far
        //
        if (count + itemsOnScreen + amountToMove > fsItemNumber) {
          var newAmountToMove = fsItemNumber - (count + itemsOnScreen);
          advanceTrack(newAmountToMove);
        } else {
          advanceTrack(amountToMove);
        }
      }
    } else {
      if (count != 0) {
        // calc if track can decrease that far
        //
        if (count - amountToMove < 0) {
          var newAmountToMove = Math.abs(0 - count);
          return decreaseTrack(newAmountToMove)
        }

        return decreaseTrack(amountToMove)
      }
    }
  };

  var advanceTrack = function (amountToMoveVal) {
    count = count + amountToMoveVal;
    currentTrackPosition = parseInt(currentTrackPosition + (fsItemWidth + fsItemGutter * 2) * amountToMoveVal);
    $(fsTrack).css('transform', 'translateX( -' + Math.abs(currentTrackPosition) + 'px )');
    hideButtons();
    if (returnedOptions.pips === true && itemsToMove >= 4) {
      modifyClickCount('forward');
    }
    // pipActiveState()
  };

  var decreaseTrack = function (amountToMoveVal) {
    count = count - amountToMoveVal;
    currentTrackPosition = parseInt(currentTrackPosition - (fsItemWidth + fsItemGutter * 2) * amountToMoveVal);
    $(fsTrack).css('transform', 'translateX( -' + Math.abs(currentTrackPosition) + 'px )');
    hideButtons();
    if (returnedOptions.pips === true && itemsToMove >= 4) {
      modifyClickCount('back');
    }
    // pipActiveState()
  };

  var modifyClickCount = function (direction) {
    if (direction === 'forward') {
      if (itemsToMove === itemsOnScreen) {
        currentPage++;
      }
    }

    if (direction === 'back') {
      currentPage--;
    }

    updatePips(currentPage);
  };

  var setUpPips = function () {
    $(fsArticle)
      .addClass('has-pips')
      .append('<div class="fs-pips"></div>');
    calcPipsPosition();

    numberOfPips = Math.round(fsItemNumber / returnedOptions.itemsOnScreen);

    for (var i = 0; i < numberOfPips; i++) {
      $(fsPips).append('<div class="fs-pip"></div>');
    }

    $(fsPips)
      .children('.fs-pip')
      .first()
      .addClass('active');
  };

  var updatePips = function (currentPage) {
    $('.fs-pip', $(fsArticle)).removeClass('active');

    $.each($('.fs-pip', $(fsArticle)), function (index, pip) {
      if (index + 1 === currentPage) {
        $(pip).addClass('active');
      }
    });
  };

  var calcPipsPosition = function () {
    fsPips = fsArticle.find('.fs-pips').css({
      width: containerWidth,
      right: trackIndent + fsItemGutter
    });

    return
  };

  // if at the start / end of the track hide the relevant button
  //
  var hideButtons = function () {
    if (count + itemsOnScreen == fsItemNumber) {
      $(fsButtonNext).css('opacity', '0');
    } else {
      $(fsButtonNext).css('opacity', '1');
    }

    if (count < 1) {
      $(fsButtonPrev).css('opacity', '0');
    } else {
      $(fsButtonPrev).css('opacity', '1');
    }
  };

  // recalculate track postion used on rsponisve
  //
  var recalcTrackPosition = function () {
    $(fsTrack).css('transform', 'translateX(' + '-' + (fsItemWidth + fsItemGutter * 2) * count + 'px )');
    currentTrackPosition = parseInt((fsItemWidth + fsItemGutter * 2) * count);
  };

  // responsive
  //
  var calcResponsive = function () {
    if (returnedOptions.responsive != null) {
      $(returnedOptions.responsive).each(function (index) {
        if (screenWidth < returnedOptions.responsive[index].breakpoint) {
          itemsOnScreen = returnedOptions.responsive[index].settings.itemsOnScreen;
          dragItemsToMove = returnedOptions.responsive[index].settings.itemsOnScreen;
          setItemWidth(returnedOptions.responsive[index].settings.itemsOnScreen);
          calcTrackWidth();
          recalcTrackPosition();
          shouldIhaveButtons(returnedOptions.responsive[index].settings.itemsOnScreen);
          // setUpButtons(returnedOptions.responsive[index].settings.itemsOnScreen)
          if (hasButtons === true) {
            calcButtonWidth();
          }
          itemsToMove = returnedOptions.responsive[index].settings.itemsToMove;
          hideButtons();
        } else if (screenWidth >= returnedOptions.responsive[0].breakpoint) {
          itemsOnScreen = returnedOptions.itemsOnScreen;
          dragItemsToMove = returnedOptions.itemsOnScreen;
          setItemWidth(itemsOnScreen);
          recalcTrackPosition();
          shouldIhaveButtons(returnedOptions.itemsOnScreen);
          if (hasButtons === true) {
            calcButtonWidth();
          }
          itemsToMove = returnedOptions.itemsToMove;
          hideButtons();
        }
      });
    } else {
      itemsOnScreen = returnedOptions.itemsOnScreen;
      dragItemsToMove = returnedOptions.itemsOnScreen;
      setItemWidth(itemsOnScreen);
      recalcTrackPosition();
      if (hasButtons === true) {
        calcButtonWidth();
      }
      itemsToMove = returnedOptions.itemsToMove;
      hideButtons();
    }
  };

  // drag
  //
  var drag = function () {
    var limit = true;
    var touchStartX = 0;
    var touchStartY = 0;
    var oldTouchX = 0;
    var oldTouchY = 0;

    // mobile drag events
    //
    $(fsTrack).bind('touchstart', function (e) {
      touchStartX = e.originalEvent.touches[0].clientX;
      touchStartY = e.originalEvent.touches[0].clientY;
      dragStarted();
    });

    $(fsTrack).bind('touchmove', function (e) {
      oldTouchX = e.originalEvent.changedTouches[0].clientX;
      oldTouchY = e.originalEvent.changedTouches[0].clientY;

      if (touchStartX > oldTouchX) {
        if (limit) {
          nextItem(dragItemsToMove);
          limit = false;
        }
      } else if (touchStartX < oldTouchX) {
        if (limit) {
          prevItem(dragItemsToMove);
          limit = false;
        }
      }
      if (returnedOptions.debug == true) {
        $('.fs-touch-event-indicator').css('left', oldTouchX);
      }
    });

    $(document).bind('touchend', function (e) {
      dragStopped(e);
    });

    // desktop drag events
    //
    if (returnedOptions.dragOnDesktop === true) {

      $(fsTrack).mousedown(function (e) {
        dragStarted();
      });

      $(document).on('mousemove', function (e) {
        dragMove(e);
      });

      $(document).mouseup(function (e) {
        dragStopped(e);
      });

      var dragMove = function (e) {
        if (dragging) {
          console.log(returnedOptions.dragThreshold);
          // setClear();
          var action = e.originalEvent.movementX < -returnedOptions.dragThreshold
            ? "left"
            : e.originalEvent.movementX > returnedOptions.dragThreshold
              ? "right"
              : "";

          if (action === "left" || action === "right") {
            if (oldPageX < e.pageX) {
              if (limit) {
                $(fsTrack).addClass('fs-being-dragged'); // disable items from being clicked
                nextItem(dragItemsToMove);
                limit = false;
              }
              oldPageX = e.pageX - pageXOffset;
            } else {
              if (limit) {
                $(fsTrack).addClass('fs-being-dragged'); // disable items from being clicked
                prevItem(dragItemsToMove);
                limit = false;
              }
              oldPageX = e.pageX + pageXOffset;
            }
          }

          // Set last X coridnate and offset to avoid triggering the wrong direction
          //
          if (returnedOptions.debug == true) {
            $('.fs-touch-event-indicator').css('left', oldPageX);
          }
          dragStopped();
          return false
        }
      };
    }

    var dragStarted = function (e) {
      dragging = true;
      return false
    };

    var dragStopped = function (e) {
      console.log('drag stopped');
      $(fsTrack).removeClass('fs-being-dragged');
      dragging = false;
      limit = true;
      e.cancelBubble = true;
    };
  };

  // init
  //
  setUpItems();
  setUpTrack();
  setUpSizingDiv();
  setItemWidth(itemsOnScreen);
  calcTrackWidth();
  calcTrackIndent();
  // setUpButtons(returnedOptions.itemsOnScreen)
  shouldIhaveButtons(returnedOptions.itemsOnScreen);
  if (hasButtons === true) {
    calcButtonWidth();
  }
  drag();
  calcResponsive();
  setArrowKeys();

  if (returnedOptions.pips === true) {
    setUpPips();
  }

  // recalulate the starting posion of first item and align with container
  //
  $(window).resize(function () {
    containerWidth = $('.fs-sizer').innerWidth();
    screenWidth = $(document).width();
    calcTrackWidth();
    calcTrackIndent();
    calcPipsPosition();
    if (hasButtons === true) {
      calcButtonWidth();
    }
    calcResponsive();
  });
};

module.exports = FlixSlider;
